
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _MOOF_FRUSTUM_HH_
#define _MOOF_FRUSTUM_HH_
	
#include <Moof/Math.hh>
#include <Moof/Plane.hh>


namespace Mf {


class Aabb;
class Sphere;

class Frustum
{
	Plane	planes_[6]; // left, right, bottom, top, near, far

public:
	typedef enum
	{
		OUTSIDE		= 0,
		INSIDE		= 1,
		INTERSECT	= 2
	} Collision;

	Frustum() {}
	Frustum(const Matrix4& modelview, const Matrix4& projection)
	{
		init(modelview, projection);
	}
	Frustum(const Matrix4& modelview, Scalar fovy, Scalar aspect,
			Scalar abutting, Scalar distant)
	{
		init(modelview, fovy, aspect, abutting, distant);
	}
	
	void init(const Matrix4& modelview, const Matrix4& projection);
	void init(const Matrix4& modelview, Scalar fovy, Scalar aspect,
			Scalar abutting, Scalar distant);

	Collision contains(const Aabb& aabb) const;
	Collision contains(const Sphere& sphere) const;
};


} // namespace Mf

#endif // _MOOF_FRUSTUM_HH_

/** vim: set ts=4 sw=4 tw=80: *************************************************/

